/*---------------------------------------------------------------------------------------------------------*/
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "NUC123Series.h"
#include "L_SPI.h"

#define SPIFLASH_STATUS_BUSY       (0x01UL)

#define RXBUFSIZE 256
extern volatile uint8_t comRbuf[512];
extern volatile uint32_t Add;
extern volatile uint8_t WriteFlash;

void SPI_Master_Init(void)
{                                
    GPIO_SPI0();			 //:PC0~3ΪSPI//en:Configure PC0~3 as SPI function
    
	DrvSPI_Init(SPI0, SPI_MASTER, SPI_TYPE0,32);
    
    SPI_SET_DIVIDER(SPI0, 1);	 //SPIclk = HCLK/((SPI_DIV+1)*2)=72/((1+1)*2)=18M
                                  
    SPI_SET_SS(SPI0, SPI_SS_LOW_TRIGGER, SPI_SW_SS);//:ԶƬѡѡ//en:Slave Select non-auto
}

/**
 * @brief       SPI send a data
 *
 * @param        ulData: data to be sent
 *				 ucLength: SPI transmit bit length, range is 0x00~0x20    
 *
 * @return      None
 *
 * @details     SPI send a data
 */
void SPI_Send_1W(uint32_t ulData, uint8_t ucLength)
{   
    SPI_Length(ucLength);
    SPI_WRITE_TX0(SPI0, ulData);                           //://en:data waiting for sending
    SPI_SET_GO(SPI0);                                             //://en: Start Transfer 
}

/* To read 8-bit status register 1 */
uint32_t SPIFLASH_ReadStatusReg1(void)
{
	SPI_SET_SS0(SPI0);
	SPI_Send_1W(0x050000, 24);
    while(SPI_IS_BUSY(SPI0) != 0); 
	SPI_CLR_SS0(SPI0);
	return SPI_GET_RX0_DATA(SPI0);

}

void SPIFLASH_WaitReady(void)
{
    while (SPIFLASH_ReadStatusReg1()&SPIFLASH_STATUS_BUSY); // check the BUSY bit
}

/**
 * @brief       Program SPI flash page0 with data 0x00,0x01, ... 0xFF.
 *
 * @param       None
 *	   
 *
 * @return      None
 *
 * @details     Program SPI flash page0 with data 0x00,0x01, ... 0xFF.
 */
void Program_Page0_Data(void)
{
    uint16_t uiCounter1 = 0;

    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(WRITER_ENABLE, 0x08);            //:дʹָ//en: Send write enable command 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive 

    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(PAGE_PROGRAM, 0x08);             //:дָ//en Write Instruction 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished

    SPI_Send_1W(Add, 24);                   //:дַ//en: Write Address 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished
    
	SPI_Length(8);

	if(WriteFlash==1)
	{
		for(uiCounter1=0; uiCounter1<256; uiCounter1++)   
	    {
			SPI_WRITE_TX0(SPI0, comRbuf[uiCounter1]);
			SPI_SET_GO(SPI0);
	        while(SPI_IS_BUSY(SPI0) != 0);           //:ȴͽ//en:waiting for sending finished
	    }
	}
	else
	{
		for(uiCounter1=0; uiCounter1<256; uiCounter1++)   
	    {
			SPI_WRITE_TX0(SPI0, comRbuf[uiCounter1+256]);
			SPI_SET_GO(SPI0);
	        while(SPI_IS_BUSY(SPI0) != 0);           //:ȴͽ//en:waiting for sending finished
	    }
	}
    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive 
                                                                                             
//    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active  
//    SPI_Send_1W(WRITER_DISABLE, 0x08);           //:ֹд//en: Write Disable
//    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished
//    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive  

	Add=Add+256;
	WriteFlash=0;
}

/**
 * @brief       Read SPI flash page0, print in Hyper-terminal via UART0 
 *
 * @param       None
 *	   
 *
 * @return      None
 *
 * @details     Read SPI flash page0, print in Hyper-terminal via UART0 
 */
void Read_Page0_Data(void)
{
    uint16_t uiCounter1 = 0;

    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(READ_DATAE, 0x08);             //:ָ//en: Read Instruction    
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished

    SPI_Send_1W(Add, 24);                 //:͵ַ//en: Write Address      
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished

    for(uiCounter1=0; uiCounter1<256; uiCounter1++)  //://en: Read Data
    {   	                                                      
        SPI_Send_1W(0xFF, 8);   					    
        while(SPI_IS_BUSY(SPI0) != 0);         //:ȴͽ//en:waiting for sending finished
//        printf("%02x,",(SPI_GET_RX0_DATA(SPI0)&0x000000FF));
//        if((uiCounter1%0x10) == 0x0F)
//        printf("\n");
        while(UART_GET_TX_EMPTY(UART0)==0);
		UART_WRITE(UART0, SPI_GET_RX0_DATA(SPI0)&0xFF); 
    }
    SPI_CLR_SS0(SPI0);                     //:ƬѡЧ//en: Set CS inactive 

	Add=Add+256;
}
    
/**
 * @brief       Chip Erase
 *
 * @param       None
 *	   
 *
 * @return      None
 *
 * @details     Chip Erase 
 */
void Erase_All_Chip(void)
{
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(WRITER_ENABLE, 0x08);          //:дʹ//en: Write Enable 
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                         //:ƬѡЧ//en: Set CS inactive 
                                 
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(CHIP_ERASE, 0x08);             //:Ƭ//en: Chip Erase
	while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                         //:ƬѡЧ//en: Set CS inactive 
  
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active  
    SPI_Send_1W(WRITER_DISABLE, 0x08);         //:дֹ//en: Write Disable
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);    
}

void SPIFLASH_SectorErase(uint32_t u32Addr)
{
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(WRITER_ENABLE, 0x08);          //:дʹ//en: Write Enable 
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                         //:ƬѡЧ//en: Set CS inactive 
    
	SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(0x20000000 | u32Addr, 32);
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                         //:ƬѡЧ//en: Set CS inactive 
  
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active  
    SPI_Send_1W(WRITER_DISABLE, 0x08);         //:дֹ//en: Write Disable
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);    
	  
}

/**
 * @brief       Set SPI transmit data length
 *
 * @param       ucLength: SPI transmit bit length, range is 0x00~0x20 
 *	   
 *
 * @return      None
 *
 * @details     Set SPI transmit data length
 */
void SPI_Length(uint8_t ucLength) 
{
    if(ucLength == 0x20)
		SPI0->CNTRL &= ~(0x1F<<3);
	else
	    SPI0->CNTRL =SPI0->CNTRL & (~(0x1F<<3))|(ucLength<<3);
}

void Program_One_Data(uint32_t addr,uint8_t data)
{

    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(WRITER_ENABLE, 0x08);            //:дʹָ//en: Send write enable command 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive 

    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(PAGE_PROGRAM, 0x08);             //:дָ//en Write Instruction 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished

    SPI_Send_1W(addr, 24);                   //:дַ//en: Write Address 
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished

    SPI_Send_1W(data, 8);
    while(SPI_IS_BUSY(SPI0) != 0);           //:ȴͽ//en:waiting for sending finished

    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive 
                                                                                             
    SPI_SET_SS0(SPI0);                           //:ƬѡЧ//en: Set CS active  
    SPI_Send_1W(WRITER_DISABLE, 0x08);           //:ֹд//en: Write Disable
    while(SPI_IS_BUSY(SPI0) != 0);               //:ȴͽ//en:waiting for sending finished
    SPI_CLR_SS0(SPI0);                           //:ƬѡЧ//en: Set CS inactive  
}

uint8_t Read_One_Data(uint32_t addr)
{
    SPI_SET_SS0(SPI0);                         //:ƬѡЧ//en: Set CS active
    SPI_Send_1W(READ_DATAE, 0x08);             //:ָ//en: Read Instruction    
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished

    SPI_Send_1W(addr, 24);                 //:͵ַ//en: Write Address      
    while(SPI_IS_BUSY(SPI0) != 0);             //:ȴͽ//en:waiting for sending finished
 	                                                      
    SPI_Send_1W(0xFF, 8);   					    
    while(SPI_IS_BUSY(SPI0) != 0);         //:ȴͽ//en:waiting for sending finished
  
    SPI_CLR_SS0(SPI0);                     //:ƬѡЧ//en: Set CS inactive 
	return 	 (SPI_GET_RX0_DATA(SPI0)&0x000000FF) ;
}